function [C,b_opt]=optrobust1(P)

a=[1 0 -29.4 0 0];
b=[33 0 980];

P=tf(b,a);

% OPTROBUST1  Optimal Robust Stabilization
%
%    OPTROBUST(P) is to design a controller for a given system P such that
%    the stability margin b_pc is maximized.
% 
%    Input "P" is the given system (created with either TF, ZPK, SS, or
%    FRD).
%
%    [C,B_OPT]=OPTROBUST1(P) returns the stabilizing controller C and the 
%    maximal stability margin B_OPT.


% Get a(s),b(s)
[btemp,atemp]=tfdata(P);
a=atemp{1,1};
b=btemp{1,1};
na=length(a);
nb=length(b);

% Get a(-s) and b(-s)
signa=mod(na,2);
signb=mod(nb,2);
am=a;
bm=b;
for i=1:2:na-1
    am(i+1)=-a(i+1);
end;
if signa==0 am=-am;
else am=am;end;

for i=1:2:nb-1
    bm(i+1)=-b(i+1);
end;
if signb==0 bm=-bm;
else bm=bm;end;
ams=tf(am,1);
bms=tf(bm,1);

% Spectral factorization
ds=spectral(a,b);
dtemp=tfdata(ds);
d=dtemp{1,1};
nd=length(d);

% Diophantine equation
ds2=ds*ds;
ptemp=zpkdata(ds2);
pole=ptemp{1,1};
xy=poleplace(P,pole,0);
[ytemp,xtemp]=tfdata(xy);
y=ytemp{1,1};
x=xtemp{1,1};
ys=tf(y,1);
xs=tf(x,1);

% Get z(-s) and T(s)
fs=xs*bms-ys*ams;
ftemp=tfdata(fs);
f=ftemp{1,1};
nf=length(f);
nz=nf-nd+1;
D=zeros(nf,nz);
for i=1:nz
    for j=i:i+nd-1
        D(j,i)=d(j-i+1);
    end
end
z=D\f';
signz=mod(nz,2);
zm=z;
for i=1:2:nz-1
    zm(i+1)=-z(i+1);
end;
if signz==0 zm=-zm';
else zm=zm';end;
Ts=tf(zm,d);

% Hankel SVD
[R_G,Sigma,u,v,U,V]=hankel(Ts);
sigma1=Sigma(1);
b_opt=1/sqrt(1+sigma1^2);
U1=U(1);
[numtem,dentem]=tfdata(U1);
num=numtem{1,1};
nn=length(num);
for i=1:nn
    if num(1)==0
        num=num(2:nn);
        nn=nn-1;
    end
end
e=num/num(1);
es=tf(e,1);

% Pole placement
des=ds*es;
detemp=tfdata(des);
de=detemp{1,1};
nde=length(de);
if nde-na+1>=na
    opt=0;
elseif nde-na+1==na-1
    opt=1;
else
    error('The stabilizing controller does not exist');
end
ptemp=zpkdata(des);
pole=ptemp{1,1};
C=poleplace(P,pole,opt);

% end optrobust1