function [C,b_opt]=optrobust2(P)

% OPTROBUST2  Optimal Robust Stabilization
%
%    OPTROBUST2(P) is to design a controller for a given system P such that
%    the stability margin b_pc is maximized.
% 
%    Input "P" is the given system (created with either TF, ZPK, SS, or
%    FRD).
%
%    [C,B_OPT]=OPTROBUST2(P) returns the stabilizing controller C and the 
%    maximal stability margin B_OPT.


% Get a(s),b(s)
[btemp,atemp]=tfdata(P);
a=atemp{1,1};
b=btemp{1,1};
na=length(a);
nb=length(b);

% Spectral factorization
ds=spectral(a,b);
dtemp=tfdata(ds);
d=dtemp{1,1};
nd=length(d);

% H-matrix computation
M=tf(a,d);
N=tf(b,d);
E1=hankel(M);
E2=hankel(N);

% Singular value decomposition
E=[E1;E2];
[u,Sigma,v]=svd(E);
sigma1=Sigma(1,1);
v1=v(:,1);
b_opt=sqrt(1-sigma1^2);
[flag,table]=routh(d);
G=[];
n=nd-1;
for i=1:n
    s=tf('s');
    nri=length(table(i+1));
    rsi=0;
    for j=1:nri
        rsi=rsi+table(i+1,j)*s^(n-i-2*(j-1));
    end 
    alpha(i)=table(i,1)/table(i+1,1);
    g=sqrt(2*alpha(i))*rsi;
    G=[G g];
end
H=G*v1/ds;
[numtem,dentem]=tfdata(H);
num=numtem{1,1};
nn=length(num);
for i=1:nn
    if num(1)==0
        num=num(2:nn);
        nn=nn-1;
    end
end
e=num/num(1);
es=tf(e,1);

% pole placement
des=ds*es;
detemp=tfdata(des);
de=detemp{1,1};
nde=length(de);
if nde-na+1>=na
    opt=0;
elseif nde-na+1==na-1
    opt=1;
else
    error('The stabilizing controller does not exist');
end
ptemp=zpkdata(des);
pole=ptemp{1,1};
C=poleplace(P,pole,opt);

% end optrobust2