function ds=spectral(a,b)

% SPECTRAL  Spectral factorization
%
%    SPECTRAL(A,B) finds the spectral factorization of a given polynomial
%    A(-s)A(s)+B(-s)B(s) for given coprime polynomials A(s) and B(s).
%
%    A, B are two vectors representing the ploynomials' coefficents. For 
%    example: if the given polynomial A(s)=$a0*s^2+a1*s+a2$ then A=[a0 a1 
%    a2].
% 
%    D is the spectral factor polynomial.


na=length(a);
nb=length(b);

% Get a(-s) and b(-s)
signa=mod(na,2);signb=mod(nb,2);
am=a;bm=b;
for i=1:2:na-1
    am(i+1)=-a(i+1);
end;
if signa==0 am=-am;
else am=am;end;

for i=1:2:nb-1
    bm(i+1)=-b(i+1);
end;
if signb==0 bm=-bm;
else bm=bm;end;

% Find roots of a(-s)a(s)+b(-s)b(s)
as=tf(a,1);ams=tf(am,1);
bs=tf(b,1);bms=tf(bm,1);

cs=as*ams+bs*bms;
ctemp=tfdata(cs);
cc=ctemp{1,1};
c=cc/abs(cc(1));
d1=roots(c);

% Find spectral factor
nd1=length(d1);
d2=zeros(nd1/2,1);
j=1;
for i=1:nd1
   if real(d1(i))<0
      d2(j)=d1(i);
      j=j+1;
   end;
end;
d3=zpk(d2,[],1);
ds=sqrt(abs(cc(1)))*tf(d3);

% end spectral